"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeModulesCollector = void 0;
const hoist_1 = require("./hoist");
const path = require("path");
const fs = require("fs");
const builder_util_1 = require("builder-util");
class NodeModulesCollector {
    constructor(rootDir) {
        this.rootDir = rootDir;
        this.allDependencies = new Map();
        this.dependencyPathMap = new Map();
        this.nodeModules = [];
    }
    transToHoisterTree(obj, key = `.`, nodes = new Map()) {
        let node = nodes.get(key);
        const name = key.match(/@?[^@]+/)[0];
        if (!node) {
            node = {
                name,
                identName: name,
                reference: key.match(/@?[^@]+@?(.+)?/)[1] || ``,
                dependencies: new Set(),
                peerNames: new Set([]),
            };
            nodes.set(key, node);
            for (const dep of (obj[key] || {}).dependencies || []) {
                node.dependencies.add(this.transToHoisterTree(obj, dep, nodes));
            }
        }
        return node;
    }
    resolvePath(filePath) {
        try {
            const stats = fs.lstatSync(filePath);
            if (stats.isSymbolicLink()) {
                return fs.realpathSync(filePath);
            }
            else {
                return filePath;
            }
        }
        catch (error) {
            builder_util_1.log.debug({ message: error.message || error.stack }, "error resolving path");
            return filePath;
        }
    }
    convertToDependencyGraph(tree) {
        const result = { ".": {} };
        const flatten = (node, parentKey = ".") => {
            var _a;
            const dependencies = node.dependencies || {};
            for (const [key, value] of Object.entries(dependencies)) {
                // Skip empty dependencies(like some optionalDependencies)
                if (Object.keys(value).length === 0) {
                    continue;
                }
                const version = value.version || "";
                const newKey = `${key}@${version}`;
                this.dependencyPathMap.set(newKey, path.normalize(this.resolvePath(value.path)));
                if (!((_a = result[parentKey]) === null || _a === void 0 ? void 0 : _a.dependencies)) {
                    result[parentKey] = { dependencies: [] };
                }
                result[parentKey].dependencies.push(newKey);
                flatten(value, newKey);
            }
        };
        flatten(tree);
        return result;
    }
    getAllDependencies(tree) {
        const dependencies = tree.dependencies || {};
        for (const [key, value] of Object.entries(dependencies)) {
            if (value.dependencies && Object.keys(value.dependencies).length > 0) {
                this.allDependencies.set(`${key}@${value.version}`, value);
                this.getAllDependencies(value);
            }
        }
    }
    async getDependenciesTree() {
        const command = this.getCommand();
        const args = this.getArgs();
        const dependencies = await (0, builder_util_1.exec)(command, args, {
            cwd: this.rootDir,
            shell: true,
        });
        const dependencyTree = JSON.parse(dependencies);
        // pnpm returns an array of dependency trees
        if (Array.isArray(dependencyTree)) {
            const tree = dependencyTree[0];
            if (tree.optionalDependencies) {
                tree.dependencies = { ...tree.dependencies, ...tree.optionalDependencies };
            }
            return tree;
        }
        // yarn and npm return a single dependency tree
        return dependencyTree;
    }
    _getNodeModules(dependencies, result) {
        if (dependencies.size === 0) {
            return;
        }
        for (const d of dependencies.values()) {
            const reference = [...d.references][0];
            const p = this.dependencyPathMap.get(`${d.name}@${reference}`);
            if (p === undefined) {
                builder_util_1.log.debug({ name: d.name, reference }, "cannot find path for dependency");
                continue;
            }
            const node = {
                name: d.name,
                version: reference,
                dir: p,
            };
            result.push(node);
            if (d.dependencies.size > 0) {
                node["dependencies"] = [];
                this._getNodeModules(d.dependencies, node["dependencies"]);
            }
        }
        result.sort((a, b) => a.name.localeCompare(b.name));
    }
    getTreeFromWorkspaces(tree) {
        if (tree.workspaces && tree.dependencies) {
            for (const [key, value] of Object.entries(tree.dependencies)) {
                if (this.rootDir.endsWith(path.normalize(key))) {
                    return value;
                }
            }
        }
        return tree;
    }
    async getNodeModules() {
        const tree = await this.getDependenciesTree();
        const realTree = this.getTreeFromWorkspaces(tree);
        this.getAllDependencies(realTree);
        this.removeNonProductionDependencie(realTree);
        const dependencyGraph = this.convertToDependencyGraph(realTree);
        const hoisterResult = (0, hoist_1.hoist)(this.transToHoisterTree(dependencyGraph), { check: true });
        this._getNodeModules(hoisterResult.dependencies, this.nodeModules);
        return this.nodeModules;
    }
}
exports.NodeModulesCollector = NodeModulesCollector;
//# sourceMappingURL=nodeModulesCollector.js.map